@@ -1,6 +1,6 @@
 /*
-** Copyright (C) 2008 George Blood Audio
-** Written by Erik de Castro Lopo <erikd@mega-nerd.com>
+** Copyright (C) 2008-2014 Erik de Castro Lopo <erikd@mega-nerd.com>
+** Copyright (C) 2008-2010 George Blood Audio
 **
 ** All rights reserved.
 **
@@ -60,14 +60,13 @@ main (int argc, char *argv [])
 	int	k ;
 
 	/* Store the program name. */
-	progname = strrchr (argv [0], '/') ;
-	progname = progname ? progname + 1 : argv [0] ;
+	progname = program_name (argv [0]) ;
 
 	/* Check if we've been asked for help. */
 	if (argc < 3 || strcmp (argv [1], "--help") == 0 || strcmp (argv [1], "-h") == 0)
 		usage_exit (progname, 0) ;
 
-	/* Clear set all fields of the struct to zero bytes. */
+	/* Set all fields of the struct to zero bytes. */
 	memset (&info, 0, sizeof (info)) ;
 
 	/* Get the time in case we need it later. */
@@ -88,7 +87,7 @@ main (int argc, char *argv [])
 			continue ;
 			} ;
 
-#define HANDLE_BEXT_ARG(cmd,field) \
+#define HANDLE_BEXT_ARG(cmd, field) \
 		if (strcmp (argv [k], cmd) == 0) \
 		{	k ++ ; \
 			if (k == argc) missing_param (argv [k - 1]) ; \
@@ -105,7 +104,7 @@ main (int argc, char *argv [])
 		HANDLE_BEXT_ARG ("--bext-coding-hist", coding_history) ;
 		HANDLE_BEXT_ARG ("--bext-time-ref", time_ref) ;
 
-#define HANDLE_STR_ARG(cmd,field) \
+#define HANDLE_STR_ARG(cmd, field) \
 	if (strcmp (argv [k], cmd) == 0) \
 	{	k ++ ; \
 		if (k == argc) missing_param (argv [k - 1]) ; \
@@ -209,7 +208,7 @@ usage_exit (const char *progname, int exit_code)
 		"    --bext-orig-date         Set the 'bext' origination date.\n"
 		"    --bext-orig-time         Set the 'bext' origination time.\n"
 		"    --bext-coding-hist       Set the 'bext' coding history.\n"
-		"    --bext-time-raf          Set the 'bext' Time ref.\n"
+		"    --bext-time-ref          Set the 'bext' Time ref.\n"
 		"\n"
 		"    --str-comment            Set the metadata comment.\n"
 		"    --str-title              Set the metadata title.\n"
@@ -235,6 +234,7 @@ usage_exit (const char *progname, int exit_code)
 		"exit with an appropriate error message.\n"
 		) ;
 
+	printf ("Using %s.\n\n", sf_version_string ()) ;
 	exit (exit_code) ;
 } /* usage_exit */
 
